/*
 * Copyright (c) 2017 Linaro Limited
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <sys/util.h>
#include <sys/printk.h>
#include <inttypes.h>
#include <drivers/pwm.h>
#include <sys/__assert.h>
#include <string.h>
#include <dk_buttons_and_leds.h>

#include "buzzer.h"
/* size of stack area used by each thread */
#define STACKSIZE 1024

/* scheduling priority used by each thread */
#define PRIORITY 7

#define SLEEP_MS 500
/*alias*/
#define LED0_NODE DT_ALIAS(led2)
#define SW0_NODE DT_ALIAS(sw0)

/*FIFO*/
struct printk_data_t {
	void *fifo_reserved; /* 1st word reserved for use by fifo */
	uint32_t conp;
};

K_FIFO_DEFINE(printk_fifo);

static struct gpio_callback button_cb;
const struct device *gpio_led;
const struct device *gpio_bt;

void print(uint32_t id){
    struct printk_data_t tx_data = { .conp = id };
    size_t size = sizeof(struct printk_data_t);
    char *mem_ptr = k_malloc(size);
    __ASSERT_NO_MSG(mem_ptr != 0);

    memcpy(mem_ptr, &tx_data, size);

    k_fifo_put(&printk_fifo, mem_ptr);

    k_msleep(SLEEP_MS);
    
}


void button_pressed(const struct device *gpio_dev, struct gpio_callback *cb,
		    uint32_t pins){

        print(1);
        ui_buzzer_set_frequency(100, 20);
        k_msleep(SLEEP_MS);
        ui_buzzer_set_frequency(0, 0);
        
        
}

void blink()
{
	int cnt = 0;
	int ret;
        
        /*led*/
	int err;

	err = dk_leds_init();
	if (err) {
		printk("Could not initialize leds, err code: %d\n", err);
	}

	err = dk_set_leds_state(1, DK_ALL_LEDS_MSK);
	if (err) {
		printk("Could not set leds state, err code: %d\n", err);
	}

        /*button*/
        struct button_handler *b_cb;
        dk_button_handler_add(&b_cb);
	err = dk_buttons_init(b_cb->cb);
	if (err) {
		printk("Could not initialize leds, err code: %d\n", err);
	}
        
        /*buzzer*/
        err = ui_buzzer_init();
	if (err) {
		printk("Could not enable buzzer, err code: %d\n", err);
		return err;
	}

	while (1) {
		err = dk_set_leds_state(cnt % 2, DK_ALL_LEDS_MSK);
                if (err) {
                        printk("Could not set leds state, err code: %d\n", err);
                }
                if (cnt % 2) {
                         print(0);
                }
                k_msleep(SLEEP_MS);
                cnt++;              
	}
}


void uart_out(void)
{
	while (1) {
		struct printk_data_t *rx_data = k_fifo_get(&printk_fifo,
							   K_FOREVER);
		printk("[Info] conponent %d turn on \n",
		       rx_data->conp);
		k_free(rx_data);
	}
}



K_THREAD_DEFINE(blink_id, STACKSIZE, blink, NULL, NULL, NULL,
		PRIORITY, 0, 0);
K_THREAD_DEFINE(uart_out_id, STACKSIZE, uart_out, NULL, NULL, NULL,
		PRIORITY, 0, 0);
